@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Evolution calendar - iCalendar file backend
  *
- * Copyright (C) 2000-2003 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  * Copyright (C) 2003 Gerg rdi
  *
  * Authors: Federico Mena-Quintero <federico@ximian.com>
@@ -31,7 +31,9 @@
 #include "e-cal-backend-contacts.h"
 
 #include <glib/gi18n-lib.h>
+#include <gconf/gconf-client.h>
 #include "libedataserver/e-xml-hash-utils.h"
+#include "libedataserver/e-flag.h"
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-util.h>
 #include <libedata-cal/e-cal-backend-util.h>
@@ -43,6 +45,13 @@
 
 static ECalBackendSyncClass *parent_class;
 
+typedef enum
+{
+	CAL_DAYS,
+	CAL_HOURS,
+	CAL_MINUTES
+} CalUnits;
+
 /* Private part of the ECalBackendContacts structure */
 struct _ECalBackendContactsPrivate {
         ESourceList  *addressbook_sources;
@@ -55,6 +64,18 @@ struct _ECalBackendContactsPrivate {
 
 	GHashTable *zones;
 	icaltimezone *default_zone;
+
+	EFlag   *init_done_flag; /* is set, when the init thread gone */
+
+	/* properties related to track alarm settings for this backend */
+	GConfClient *conf_client;
+	guint notifyid1;
+	guint notifyid2;
+	guint notifyid3;
+	guint update_alarms_id;
+	gboolean alarm_enabled;
+	gint alarm_interval;
+	CalUnits alarm_units;
 };
 
 typedef struct _BookRecord {
@@ -79,8 +100,8 @@ static ECalComponent * create_anniversary (ECalBackendContacts *cbc, EContact *c
 static void contacts_changed_cb (EBookView *book_view, const GList *contacts, gpointer user_data);
 static void contacts_added_cb   (EBookView *book_view, const GList *contacts, gpointer user_data);
 static void contacts_removed_cb (EBookView *book_view, const GList *contact_ids, gpointer user_data);
-static ECalBackendSyncStatus
-e_cal_backend_contacts_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj);
+static ECalBackendSyncStatus e_cal_backend_contacts_add_timezone (ECalBackendSync *backend, EDataCal *cal, const gchar *tzobj);
+static void setup_alarm (ECalBackendContacts *cbc, ECalComponent *comp);
 
 /* BookRecord methods */
 static BookRecord *
@@ -91,14 +112,22 @@ book_record_new (ECalBackendContacts *cbc, ESource *source)
         EBookQuery *query;
         EBookView  *book_view;
         BookRecord *br;
+	GError     *error = NULL;
 
 	book = e_book_new (source, NULL);
-        e_book_open (book, TRUE, NULL);
+        if (!book || !e_book_open (book, TRUE, &error) || error) {
+		g_object_unref (book);
+		if (error) {
+			g_warning ("%s: Failed to open book, error: %s", G_STRFUNC, error->message);
+			g_error_free (error);
+		}
+		return NULL;
+	}
 
         /* Create book view */
-        fields = g_list_append (fields, (char*)e_contact_field_name (E_CONTACT_FILE_AS));
-        fields = g_list_append (fields, (char*)e_contact_field_name (E_CONTACT_BIRTH_DATE));
-        fields = g_list_append (fields, (char*)e_contact_field_name (E_CONTACT_ANNIVERSARY));
+        fields = g_list_append (fields, (gchar *)e_contact_field_name (E_CONTACT_FILE_AS));
+        fields = g_list_append (fields, (gchar *)e_contact_field_name (E_CONTACT_BIRTH_DATE));
+        fields = g_list_append (fields, (gchar *)e_contact_field_name (E_CONTACT_ANNIVERSARY));
         query = e_book_query_any_field_contains ("");
 
         if (!e_book_get_book_view (book, query, fields, -1, &book_view, NULL)) {
@@ -140,9 +169,9 @@ static ContactRecord *
 contact_record_new (ECalBackendContacts *cbc, EContact *contact)
 {
         ContactRecord *cr = g_new0 (ContactRecord, 1);
-	char *comp_str;
+	gchar *comp_str;
 
-        cr->cbc = g_object_ref (cbc);
+        cr->cbc = cbc;
         cr->contact = contact;
         cr->comp_birthday = create_birthday (cbc, contact);
         cr->comp_anniversary = create_anniversary (cbc, contact);
@@ -170,7 +199,7 @@ contact_record_new (ECalBackendContacts *cbc, EContact *contact)
 static void
 contact_record_free (ContactRecord *cr)
 {
-        char *comp_str;
+        gchar *comp_str;
 	ECalComponentId *id;
 
         g_object_unref (G_OBJECT (cr->contact));
@@ -198,7 +227,6 @@ contact_record_free (ContactRecord *cr)
 		g_object_unref (G_OBJECT (cr->comp_anniversary));
 	}
 
-	g_object_unref (cr->cbc);
         g_free (cr);
 }
 
@@ -214,7 +242,7 @@ contact_record_cb_new (ECalBackendContacts *cbc, ECalBackendSExp *sexp)
 {
         ContactRecordCB *cb_data = g_new (ContactRecordCB, 1);
 
-        cb_data->cbc = g_object_ref (cbc);
+        cb_data->cbc = cbc;
         cb_data->sexp = sexp;
         cb_data->result = NULL;
 
@@ -237,12 +265,12 @@ contact_record_cb (gpointer key, gpointer value, gpointer user_data)
         ContactRecord   *record = value;
 
         if (record->comp_birthday && e_cal_backend_sexp_match_comp (cb_data->sexp, record->comp_birthday, E_CAL_BACKEND (cb_data->cbc))) {
-                char * comp_str = e_cal_component_get_as_string (record->comp_birthday);
+                gchar * comp_str = e_cal_component_get_as_string (record->comp_birthday);
                 cb_data->result = g_list_append (cb_data->result, comp_str);
         }
 
         if (record->comp_anniversary && e_cal_backend_sexp_match_comp (cb_data->sexp, record->comp_anniversary, E_CAL_BACKEND (cb_data->cbc))) {
-                char * comp_str = e_cal_component_get_as_string (record->comp_anniversary);
+                gchar * comp_str = e_cal_component_get_as_string (record->comp_anniversary);
                 cb_data->result = g_list_append (cb_data->result, comp_str);
         }
 }
@@ -252,7 +280,10 @@ static void
 add_source (ECalBackendContacts *cbc, ESource *source)
 {
         BookRecord *br = book_record_new (cbc, source);
-        const char *uid = e_source_peek_uid (source);
+        const gchar *uid = e_source_peek_uid (source);
+
+	if (!br)
+		return;
 
         g_hash_table_insert (cbc->priv->addressbooks, g_strdup (uid), br);
 }
@@ -271,7 +302,7 @@ static void
 source_removed_cb (ESourceGroup *group, ESource *source, gpointer user_data)
 {
         ECalBackendContacts *cbc = E_CAL_BACKEND_CONTACTS (user_data);
-        const char          *uid = e_source_peek_uid (source);
+        const gchar          *uid = e_source_peek_uid (source);
 
         g_return_if_fail (cbc);
 
@@ -315,7 +346,7 @@ source_group_removed_cb (ESourceList *source_list, ESourceGroup *group, gpointer
         /* Unload all address books from this group */
         for (i = e_source_group_peek_sources (group); i; i = i->next) {
                 ESource *source = E_SOURCE (i->data);
-                const char *uid = e_source_peek_uid (source);
+                const gchar *uid = e_source_peek_uid (source);
 
                 g_hash_table_remove (cbc->priv->addressbooks, uid);
         }
@@ -331,12 +362,12 @@ contacts_changed_cb (EBookView *book_view, const GList *contacts, gpointer user_
 
         for (i = contacts; i; i = i->next) {
                 EContact *contact = E_CONTACT (i->data);
-                const char *uid = e_contact_get_const (contact, E_CONTACT_UID);
+                const gchar *uid = e_contact_get_const (contact, E_CONTACT_UID);
 
                 /* Because this is a change of contact, then always remove old tracked data
 		   and if possible, add with (possibly) new values.
 		*/
-		g_hash_table_remove (cbc->priv->tracked_contacts, (char *)uid);
+		g_hash_table_remove (cbc->priv->tracked_contacts, (gchar *)uid);
 
                 if (e_contact_get (contact, E_CONTACT_BIRTH_DATE) ||
                     e_contact_get (contact, E_CONTACT_ANNIVERSARY)) {
@@ -363,7 +394,7 @@ contacts_added_cb (EBookView *book_view, const GList *contacts, gpointer user_da
 
                 if (birthday || anniversary) {
                         ContactRecord *cr = contact_record_new (cbc, contact);
-                        const char    *uid = e_contact_get_const (contact, E_CONTACT_UID);
+                        const gchar    *uid = e_contact_get_const (contact, E_CONTACT_UID);
 
                         g_hash_table_insert (cbc->priv->tracked_contacts, g_strdup (uid), cr);
                 }
@@ -403,9 +434,164 @@ cdate_to_icaltime (EContactDate *cdate)
 	return ret;
 }
 
+static void
+manage_comp_alarm_update (ECalBackendContacts *cbc, ECalComponent *comp)
+{
+	gchar *old_comp, *new_comp;
+
+	g_return_if_fail (cbc != NULL);
+	g_return_if_fail (comp != NULL);
+
+	old_comp = e_cal_component_get_as_string (comp);
+	setup_alarm (cbc, comp);
+	new_comp = e_cal_component_get_as_string (comp);
+
+	/* check if component changed and notify if so */
+	if (old_comp && new_comp && !g_str_equal (old_comp, new_comp))
+		e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbc), old_comp, new_comp);
+
+	g_free (old_comp);
+	g_free (new_comp);
+}
+
+static void
+update_alarm_cb (gpointer key, gpointer value, gpointer user_data)
+{
+	ECalBackendContacts *cbc = user_data;
+        ContactRecord   *record = value;
+
+	g_return_if_fail (cbc != NULL);
+	g_return_if_fail (record != NULL);
+
+        if (record->comp_birthday)
+		manage_comp_alarm_update (cbc, record->comp_birthday);
+
+        if (record->comp_anniversary)
+                manage_comp_alarm_update (cbc, record->comp_anniversary);
+}
+
+static gboolean
+update_tracked_alarms_cb (gpointer user_data)
+{
+	ECalBackendContacts *cbc = user_data;
+
+	g_return_val_if_fail (cbc != NULL, FALSE);
+
+	g_hash_table_foreach (cbc->priv->tracked_contacts, update_alarm_cb, cbc);
+	cbc->priv->update_alarms_id = 0;
+
+	return FALSE;
+}
+
+static void
+alarm_config_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	ECalBackendContacts *cbc = user_data;
+
+	g_return_if_fail (cbc != NULL);
+
+	setup_alarm (cbc, NULL);
+
+	if (!cbc->priv->update_alarms_id)
+		cbc->priv->update_alarms_id = g_idle_add (update_tracked_alarms_cb, cbc);
+}
+
+/* When called with NULL, then just refresh local static variables on setup change from the user. */
+static void
+setup_alarm (ECalBackendContacts *cbc, ECalComponent *comp)
+{
+	ECalComponentAlarm *alarm;
+	ECalComponentAlarmTrigger trigger;
+	ECalComponentText summary;
+
+	g_return_if_fail (cbc != NULL);
+
+	if (!comp || cbc->priv->alarm_interval == -1) {
+		gchar *str;
+
+		#define BA_CONF_DIR		"/apps/evolution/calendar/other"
+		#define BA_CONF_ENABLED		BA_CONF_DIR "/use_ba_reminder"
+		#define BA_CONF_INTERVAL	BA_CONF_DIR "/ba_reminder_interval"
+		#define BA_CONF_UNITS		BA_CONF_DIR "/ba_reminder_units"
+
+		if (cbc->priv->alarm_interval == -1) {
+			/* initial setup, hook callback for changes too */
+			gconf_client_add_dir (cbc->priv->conf_client, BA_CONF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
+			cbc->priv->notifyid1 = gconf_client_notify_add (cbc->priv->conf_client, BA_CONF_ENABLED,  alarm_config_changed_cb, cbc, NULL, NULL);
+			cbc->priv->notifyid2 = gconf_client_notify_add (cbc->priv->conf_client, BA_CONF_INTERVAL, alarm_config_changed_cb, cbc, NULL, NULL);
+			cbc->priv->notifyid3 = gconf_client_notify_add (cbc->priv->conf_client, BA_CONF_UNITS,    alarm_config_changed_cb, cbc, NULL, NULL);
+		}
+
+		cbc->priv->alarm_enabled = gconf_client_get_bool (cbc->priv->conf_client, BA_CONF_ENABLED, NULL);
+		cbc->priv->alarm_interval = gconf_client_get_int (cbc->priv->conf_client, BA_CONF_INTERVAL, NULL);
+
+		str = gconf_client_get_string (cbc->priv->conf_client, BA_CONF_UNITS, NULL);
+		if (str && !strcmp (str, "days"))
+			cbc->priv->alarm_units = CAL_DAYS;
+		else if (str && !strcmp (str, "hours"))
+			cbc->priv->alarm_units = CAL_HOURS;
+		else
+			cbc->priv->alarm_units = CAL_MINUTES;
+
+		g_free (str);
+
+		if (cbc->priv->alarm_interval <= 0)
+			cbc->priv->alarm_interval = 1;
+
+		if (!comp)
+			return;
+
+		#undef BA_CONF_DIR
+		#undef BA_CONF_ENABLED
+		#undef BA_CONF_INTERVAL
+		#undef BA_CONF_UNITS
+	}
+
+	/* ensure no alarms left */
+	e_cal_component_remove_all_alarms (comp);
+
+	/* do not want alarms, return */
+	if (!cbc->priv->alarm_enabled)
+		return;
+
+	alarm = e_cal_component_alarm_new ();
+	e_cal_component_get_summary (comp, &summary);
+	e_cal_component_alarm_set_description (alarm, &summary);
+	e_cal_component_alarm_set_action (alarm, E_CAL_COMPONENT_ALARM_DISPLAY);
+
+	trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
+
+	memset (&trigger.u.rel_duration, 0, sizeof (trigger.u.rel_duration));
+
+	trigger.u.rel_duration.is_neg = TRUE;
+
+	switch (cbc->priv->alarm_units) {
+	case CAL_MINUTES:
+		trigger.u.rel_duration.minutes = cbc->priv->alarm_interval;
+		break;
+
+	case CAL_HOURS:
+		trigger.u.rel_duration.hours = cbc->priv->alarm_interval;
+		break;
+
+	case CAL_DAYS:
+		trigger.u.rel_duration.days = cbc->priv->alarm_interval;
+		break;
+
+	default:
+		g_warning ("%s: wrong units %d\n", G_STRFUNC, cbc->priv->alarm_units);
+		e_cal_component_alarm_free (alarm);
+		return;
+	}
+
+	e_cal_component_alarm_set_trigger (alarm, trigger);
+	e_cal_component_add_alarm (comp, alarm);
+	e_cal_component_alarm_free (alarm);
+}
+
 /* Contact -> Event creator */
 static ECalComponent *
-create_component (ECalBackendContacts *cbc, const char *uid, EContactDate *cdate, const char *summary)
+create_component (ECalBackendContacts *cbc, const gchar *uid, EContactDate *cdate, const gchar *summary)
 {
         ECalComponent             *cal_comp;
 	ECalComponentText          comp_summary;
@@ -467,6 +653,9 @@ create_component (ECalBackendContacts *cbc, const char *uid, EContactDate *cdate
 	/* Birthdays/anniversaries are shown as free time */
 	e_cal_component_set_transparency (cal_comp, E_CAL_COMPONENT_TRANSP_TRANSPARENT);
 
+	/* setup alarms if required */
+	setup_alarm (cbc, cal_comp);
+
         /* Don't forget to call commit()! */
 	e_cal_component_commit_sequence (cal_comp);
 
@@ -478,14 +667,14 @@ create_birthday (ECalBackendContacts *cbc, EContact *contact)
 {
         EContactDate  *cdate;
         ECalComponent *cal_comp;
-	char          *summary;
-        const char    *name;
-        char *uid;
+	gchar          *summary;
+        const gchar    *name;
+        gchar *uid;
 
         cdate = e_contact_get (contact, E_CONTACT_BIRTH_DATE);
         name = e_contact_get_const (contact, E_CONTACT_FILE_AS);
 
-	uid = g_strdup_printf ("%s%s", (char *) e_contact_get_const (contact, E_CONTACT_UID), BIRTHDAY_UID_EXT);
+	uid = g_strdup_printf ("%s%s", (gchar *) e_contact_get_const (contact, E_CONTACT_UID), BIRTHDAY_UID_EXT);
         summary = g_strdup_printf (_("Birthday: %s"), name);
 
         cal_comp = create_component (cbc, uid, cdate, summary);
@@ -502,14 +691,14 @@ create_anniversary (ECalBackendContacts *cbc, EContact *contact)
 {
         EContactDate  *cdate;
         ECalComponent *cal_comp;
-	char          *summary;
-        const char    *name;
-        char *uid;
+	gchar          *summary;
+        const gchar    *name;
+        gchar *uid;
 
         cdate = e_contact_get (contact, E_CONTACT_ANNIVERSARY);
         name = e_contact_get_const (contact, E_CONTACT_FILE_AS);
 
-	uid = g_strdup_printf ("%s%s", (char *) e_contact_get_const (contact, E_CONTACT_UID), ANNIVERSARY_UID_EXT);
+	uid = g_strdup_printf ("%s%s", (gchar *) e_contact_get_const (contact, E_CONTACT_UID), ANNIVERSARY_UID_EXT);
         summary = g_strdup_printf (_("Anniversary: %s"), name);
 
         cal_comp = create_component (cbc, uid, cdate, summary);
@@ -528,7 +717,7 @@ create_anniversary (ECalBackendContacts *cbc, EContact *contact)
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_cal_address (ECalBackendSync *backend, EDataCal *cal,
-					char **address)
+					gchar **address)
 {
 	/* A contact backend has no particular email address associated
 	 * with it (although that would be a useful feature some day).
@@ -540,7 +729,7 @@ e_cal_backend_contacts_get_cal_address (ECalBackendSync *backend, EDataCal *cal,
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_ldap_attribute (ECalBackendSync *backend, EDataCal *cal,
-					   char **attribute)
+					   gchar **attribute)
 {
 	*attribute = NULL;
 
@@ -549,7 +738,7 @@ e_cal_backend_contacts_get_ldap_attribute (ECalBackendSync *backend, EDataCal *c
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_alarm_email_address (ECalBackendSync *backend, EDataCal *cal,
-						char **address)
+						gchar **address)
 {
 	/* A contact backend has no particular email address associated
 	 * with it (although that would be a useful feature some day).
@@ -561,9 +750,9 @@ e_cal_backend_contacts_get_alarm_email_address (ECalBackendSync *backend, EDataC
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_static_capabilities (ECalBackendSync *backend, EDataCal *cal,
-						char **capabilities)
+						gchar **capabilities)
 {
- 	*capabilities = NULL;
+	*capabilities = NULL;
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -577,20 +766,20 @@ e_cal_backend_contacts_remove (ECalBackendSync *backend, EDataCal *cal)
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_default_object (ECalBackendSync *backend, EDataCal *cal,
-					   char **object)
+					   gchar **object)
 {
 	return GNOME_Evolution_Calendar_UnsupportedMethod;
 }
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_object (ECalBackendSync *backend, EDataCal *cal,
-				   const char *uid, const char *rid,
-				   char **object)
+				   const gchar *uid, const gchar *rid,
+				   gchar **object)
 {
         ECalBackendContacts *cbc = E_CAL_BACKEND_CONTACTS (backend);
         ECalBackendContactsPrivate *priv = cbc->priv;
 	ContactRecord *record;
-	char *real_uid;
+	gchar *real_uid;
 
 	if (!uid)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
@@ -635,7 +824,7 @@ e_cal_backend_contacts_get_free_busy (ECalBackendSync *backend, EDataCal *cal,
 
 	icalcomponent *vfb = icalcomponent_new_vfreebusy ();
 	icaltimezone *utc_zone = icaltimezone_get_utc_timezone ();
-	char *calobj;
+	gchar *calobj;
 
 #if 0
 	icalproperty *prop;
@@ -653,7 +842,7 @@ e_cal_backend_contacts_get_free_busy (ECalBackendSync *backend, EDataCal *cal,
 	icalcomponent_set_dtstart (vfb, icaltime_from_timet_with_zone (start, FALSE, utc_zone));
 	icalcomponent_set_dtend (vfb, icaltime_from_timet_with_zone (end, FALSE, utc_zone));
 
-	calobj = icalcomponent_as_ical_string (vfb);
+	calobj = icalcomponent_as_ical_string_r (vfb);
 	*freebusy = g_list_append (NULL, calobj);
 	icalcomponent_free (vfb);
 
@@ -663,7 +852,7 @@ e_cal_backend_contacts_get_free_busy (ECalBackendSync *backend, EDataCal *cal,
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_changes (ECalBackendSync *backend, EDataCal *cal,
-				    const char *change_id,
+				    const gchar *change_id,
 				    GList **adds, GList **modifies, GList **deletes)
 {
 	/* WRITE ME */
@@ -672,7 +861,7 @@ e_cal_backend_contacts_get_changes (ECalBackendSync *backend, EDataCal *cal,
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_discard_alarm (ECalBackendSync *backend, EDataCal *cal,
-				      const char *uid, const char *auid)
+				      const gchar *uid, const gchar *auid)
 {
 	/* WRITE ME */
 	return GNOME_Evolution_Calendar_Success;
@@ -680,14 +869,14 @@ e_cal_backend_contacts_discard_alarm (ECalBackendSync *backend, EDataCal *cal,
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_receive_objects (ECalBackendSync *backend, EDataCal *cal,
-					const char *calobj)
+					const gchar *calobj)
 {
 	return GNOME_Evolution_Calendar_PermissionDenied;
 }
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_send_objects (ECalBackendSync *backend, EDataCal *cal,
-				     const char *calobj, GList **users, char **modified_calobj)
+				     const gchar *calobj, GList **users, gchar **modified_calobj)
 {
 	*users = NULL;
 	*modified_calobj = NULL;
@@ -721,15 +910,40 @@ e_cal_backend_contacts_is_read_only (ECalBackendSync *backend, EDataCal *cal,
 	return GNOME_Evolution_Calendar_Success;
 }
 
+static gpointer
+init_sources_cb (ECalBackendContacts *cbc)
+{
+        ECalBackendContactsPrivate *priv;
+        GSList *i;
+
+	g_return_val_if_fail (cbc != NULL, FALSE);
+
+	priv = cbc->priv;
+
+	/* Create address books for existing sources */
+        for (i = e_source_list_peek_groups (priv->addressbook_sources); i; i = i->next) {
+                ESourceGroup *source_group = E_SOURCE_GROUP (i->data);
+
+                source_group_added_cb (priv->addressbook_sources, source_group, cbc);
+        }
+
+        /* Listen for source list changes */
+        g_signal_connect (priv->addressbook_sources, "group_added", G_CALLBACK (source_group_added_cb), cbc);
+        g_signal_connect (priv->addressbook_sources, "group_removed", G_CALLBACK (source_group_removed_cb), cbc);
+
+	e_flag_set (priv->init_done_flag);
+
+	return NULL;
+}
+
 static ECalBackendSyncStatus
 e_cal_backend_contacts_open (ECalBackendSync *backend, EDataCal *cal,
 			     gboolean only_if_exists,
-			     const char *username, const char *password)
+			     const gchar *username, const gchar *password)
 {
         ECalBackendContacts *cbc = E_CAL_BACKEND_CONTACTS (backend);
         ECalBackendContactsPrivate *priv = cbc->priv;
-
-        GSList *i;
+	GError *error = NULL;
 
         if (priv->addressbook_loaded)
                 return GNOME_Evolution_Calendar_Success;
@@ -743,18 +957,18 @@ e_cal_backend_contacts_open (ECalBackendSync *backend, EDataCal *cal,
 		g_hash_table_insert (priv->zones, g_strdup (icaltimezone_get_tzid (zone)), zone);
 	}
 
-	/* Create address books for existing sources */
-        for (i = e_source_list_peek_groups (priv->addressbook_sources); i; i = i->next) {
-                ESourceGroup *source_group = E_SOURCE_GROUP (i->data);
+	/* initialize addressbook sources in new thread to make this function quick as much as possible */
+	if (!g_thread_create ((GThreadFunc)init_sources_cb, cbc, FALSE, &error)) {
+		e_flag_set (priv->init_done_flag);
+		g_warning ("%s: Cannot create thread to initialize sources! (%s)", G_STRFUNC, error ? error->message : "Unknown error");
+		if (error)
+			g_error_free (error);
 
-                source_group_added_cb (priv->addressbook_sources, source_group, cbc);
-        }
-
-        /* Listen for source list changes */
-        g_signal_connect (priv->addressbook_sources, "group_added", G_CALLBACK (source_group_added_cb), cbc);
-        g_signal_connect (priv->addressbook_sources, "group_removed", G_CALLBACK (source_group_removed_cb), cbc);
+		return GNOME_Evolution_Calendar_OtherError;
+	}
 
         priv->addressbook_loaded = TRUE;
+
         return GNOME_Evolution_Calendar_Success;
 }
 
@@ -768,7 +982,7 @@ e_cal_backend_contacts_is_loaded (ECalBackend *backend)
 }
 
 static ECalBackendSyncStatus
-e_cal_backend_contacts_get_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzid, char **object)
+e_cal_backend_contacts_get_timezone (ECalBackendSync *backend, EDataCal *cal, const gchar *tzid, gchar **object)
 {
 	ECalBackendContacts *cbcontacts;
 	ECalBackendContactsPrivate *priv;
@@ -788,20 +1002,20 @@ e_cal_backend_contacts_get_timezone (ECalBackendSync *backend, EDataCal *cal, co
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
 
-	*object = icalcomponent_as_ical_string (icalcomp);
+	*object = icalcomponent_as_ical_string_r (icalcomp);
 
 	return GNOME_Evolution_Calendar_Success;
 }
 
 /* Add_timezone handler for the file backend */
 static ECalBackendSyncStatus
-e_cal_backend_contacts_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj)
+e_cal_backend_contacts_add_timezone (ECalBackendSync *backend, EDataCal *cal, const gchar *tzobj)
 {
 	ECalBackendContacts *cbcontacts;
 	ECalBackendContactsPrivate *priv;
 	icalcomponent *tz_comp;
 	icaltimezone *zone;
-	char *tzid;
+	const gchar *tzid;
 
 	cbcontacts = (ECalBackendContacts *) backend;
 
@@ -833,7 +1047,7 @@ e_cal_backend_contacts_add_timezone (ECalBackendSync *backend, EDataCal *cal, co
 }
 
 static ECalBackendSyncStatus
-e_cal_backend_contacts_set_default_zone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj)
+e_cal_backend_contacts_set_default_zone (ECalBackendSync *backend, EDataCal *cal, const gchar *tzobj)
 {
 	icalcomponent *tz_comp;
 	ECalBackendContacts *cbcontacts;
@@ -865,7 +1079,7 @@ e_cal_backend_contacts_set_default_zone (ECalBackendSync *backend, EDataCal *cal
 
 static ECalBackendSyncStatus
 e_cal_backend_contacts_get_object_list (ECalBackendSync *backend, EDataCal *cal,
-					const char *sexp_string, GList **objects)
+					const gchar *sexp_string, GList **objects)
 {
         ECalBackendContacts *cbc = E_CAL_BACKEND_CONTACTS (backend);
         ECalBackendContactsPrivate *priv = cbc->priv;
@@ -919,7 +1133,7 @@ e_cal_backend_contacts_internal_get_default_timezone (ECalBackend *backend)
 }
 
 static icaltimezone *
-e_cal_backend_contacts_internal_get_timezone (ECalBackend *backend, const char *tzid)
+e_cal_backend_contacts_internal_get_timezone (ECalBackend *backend, const gchar *tzid)
 {
         ECalBackendContacts *cbc = E_CAL_BACKEND_CONTACTS (backend);
 
@@ -948,6 +1162,17 @@ e_cal_backend_contacts_finalize (GObject *object)
 	cbc = E_CAL_BACKEND_CONTACTS (object);
 	priv = cbc->priv;
 
+	if (priv->init_done_flag) {
+		e_flag_wait (priv->init_done_flag);
+		e_flag_free (priv->init_done_flag);
+		priv->init_done_flag = NULL;
+	}
+
+	if (priv->update_alarms_id) {
+		g_source_remove (priv->update_alarms_id);
+		priv->update_alarms_id = 0;
+	}
+
 	if (priv->default_zone && priv->default_zone != icaltimezone_get_utc_timezone ()) {
 		icaltimezone_free (priv->default_zone, 1);
 	}
@@ -957,6 +1182,14 @@ e_cal_backend_contacts_finalize (GObject *object)
 	g_hash_table_destroy (priv->addressbooks);
         g_hash_table_destroy (priv->tracked_contacts);
         g_hash_table_destroy (priv->zones);
+	if (priv->notifyid1)
+		gconf_client_notify_remove (priv->conf_client, priv->notifyid1);
+	if (priv->notifyid2)
+		gconf_client_notify_remove (priv->conf_client, priv->notifyid2);
+	if (priv->notifyid3)
+		gconf_client_notify_remove (priv->conf_client, priv->notifyid3);
+
+	g_object_unref (priv->conf_client);
 
 	g_free (priv);
 	cbc->priv = NULL;
@@ -982,6 +1215,15 @@ e_cal_backend_contacts_init (ECalBackendContacts *cbc, ECalBackendContactsClass
 
 	priv->zones = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, free_zone);
 	priv->default_zone = icaltimezone_get_utc_timezone ();
+	priv->init_done_flag = e_flag_new ();
+	priv->conf_client = gconf_client_get_default ();
+	priv->notifyid1 = 0;
+	priv->notifyid2 = 0;
+	priv->notifyid3 = 0;
+	priv->update_alarms_id = 0;
+	priv->alarm_enabled = FALSE;
+	priv->alarm_interval = -1;
+	priv->alarm_units = CAL_MINUTES;
 
 	cbc->priv = priv;
 
@@ -989,7 +1231,7 @@ e_cal_backend_contacts_init (ECalBackendContacts *cbc, ECalBackendContactsClass
 }
 
 static ECalBackendSyncStatus
-e_cal_backend_contacts_create_object (ECalBackendSync *backend, EDataCal *cal, char **calobj, char **uid)
+e_cal_backend_contacts_create_object (ECalBackendSync *backend, EDataCal *cal, gchar **calobj, gchar **uid)
 {
         ECalBackendContacts *cbcontacts;
         ECalBackendContactsPrivate *priv;
@@ -1018,16 +1260,16 @@ e_cal_backend_contacts_class_init (ECalBackendContactsClass *class)
 
 	sync_class->is_read_only_sync = e_cal_backend_contacts_is_read_only;
 	sync_class->get_cal_address_sync = e_cal_backend_contacts_get_cal_address;
- 	sync_class->get_alarm_email_address_sync = e_cal_backend_contacts_get_alarm_email_address;
- 	sync_class->get_ldap_attribute_sync = e_cal_backend_contacts_get_ldap_attribute;
- 	sync_class->get_static_capabilities_sync = e_cal_backend_contacts_get_static_capabilities;
+	sync_class->get_alarm_email_address_sync = e_cal_backend_contacts_get_alarm_email_address;
+	sync_class->get_ldap_attribute_sync = e_cal_backend_contacts_get_ldap_attribute;
+	sync_class->get_static_capabilities_sync = e_cal_backend_contacts_get_static_capabilities;
 	sync_class->open_sync = e_cal_backend_contacts_open;
 	sync_class->remove_sync = e_cal_backend_contacts_remove;
 	sync_class->create_object_sync = e_cal_backend_contacts_create_object;
 	sync_class->discard_alarm_sync = e_cal_backend_contacts_discard_alarm;
 	sync_class->receive_objects_sync = e_cal_backend_contacts_receive_objects;
 	sync_class->send_objects_sync = e_cal_backend_contacts_send_objects;
- 	sync_class->get_default_object_sync = e_cal_backend_contacts_get_default_object;
+	sync_class->get_default_object_sync = e_cal_backend_contacts_get_default_object;
 	sync_class->get_object_sync = e_cal_backend_contacts_get_object;
 	sync_class->get_object_list_sync = e_cal_backend_contacts_get_object_list;
 	sync_class->get_timezone_sync = e_cal_backend_contacts_get_timezone;
@@ -1044,7 +1286,6 @@ e_cal_backend_contacts_class_init (ECalBackendContactsClass *class)
 	backend_class->internal_get_timezone = e_cal_backend_contacts_internal_get_timezone;
 }
 
-
 /**
  * e_cal_backend_contacts_get_type:
  * @void:
