@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001-2003 Ximian, Inc. (www.ximian.com)
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -30,6 +30,7 @@
 #include "camel-folder-search.h"
 #include "camel-mime-message.h"
 #include "camel-multipart.h"
+#include "camel-string-utils.h"
 
 #define d(x)
 
@@ -48,22 +49,22 @@ static CamelFolderClass *parent_class = NULL;
 
 static void digest_refresh_info (CamelFolder *folder, CamelException *ex);
 static void digest_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
-static const char *digest_get_full_name (CamelFolder *folder);
+static const gchar *digest_get_full_name (CamelFolder *folder);
 static void digest_expunge (CamelFolder *folder, CamelException *ex);
 
 /* message manipulation */
 static CamelMimeMessage *digest_get_message (CamelFolder *folder, const gchar *uid,
 					     CamelException *ex);
 static void digest_append_message (CamelFolder *folder, CamelMimeMessage *message,
-				   const CamelMessageInfo *info, char **appended_uid, CamelException *ex);
+				   const CamelMessageInfo *info, gchar **appended_uid, CamelException *ex);
 static void digest_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 					 CamelFolder *dest, GPtrArray **transferred_uids,
 					 gboolean delete_originals, CamelException *ex);
 
-static GPtrArray *digest_search_by_expression (CamelFolder *folder, const char *expression,
+static GPtrArray *digest_search_by_expression (CamelFolder *folder, const gchar *expression,
 					       CamelException *ex);
 
-static GPtrArray *digest_search_by_uids (CamelFolder *folder, const char *expression,
+static GPtrArray *digest_search_by_uids (CamelFolder *folder, const gchar *expression,
 					 GPtrArray *uids, CamelException *ex);
 
 static void digest_search_free (CamelFolder *folder, GPtrArray *result);
@@ -154,7 +155,7 @@ multipart_contains_message_parts (CamelMultipart *multipart)
 	gboolean has_message_parts = FALSE;
 	CamelDataWrapper *wrapper;
 	CamelMimePart *part;
-	int i, parts;
+	gint i, parts;
 
 	parts = camel_multipart_get_number (multipart);
 	for (i = 0; i < parts && !has_message_parts; i++) {
@@ -171,16 +172,17 @@ multipart_contains_message_parts (CamelMultipart *multipart)
 }
 
 static void
-digest_add_multipart (CamelFolder *folder, CamelMultipart *multipart, const char *preuid)
+digest_add_multipart (CamelFolder *folder, CamelMultipart *multipart, const gchar *preuid)
 {
 	CamelDataWrapper *wrapper;
 	CamelMessageInfo *info;
 	CamelMimePart *part;
-	int parts, i;
-	char *uid;
+	gint parts, i;
+	gchar *uid;
 
 	parts = camel_multipart_get_number (multipart);
 	for (i = 0; i < parts; i++) {
+		gchar *tmp;
 		part = camel_multipart_get_part (multipart, i);
 
 		wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
@@ -195,8 +197,10 @@ digest_add_multipart (CamelFolder *folder, CamelMultipart *multipart, const char
 		}
 
 		info = camel_folder_summary_info_new_from_message (folder->summary, CAMEL_MIME_MESSAGE (wrapper));
-		g_free(info->uid);
-		info->uid = g_strdup_printf ("%s%d", preuid, i);
+		camel_pstring_free(info->uid);
+		tmp = g_strdup_printf ("%s%d", preuid, i);
+		info->uid = camel_pstring_strdup (tmp);
+		g_free(tmp);
 		camel_folder_summary_add (folder->summary, info);
 	}
 }
@@ -255,7 +259,7 @@ digest_expunge (CamelFolder *folder, CamelException *ex)
 	/* no-op */
 }
 
-static const char *
+static const gchar *
 digest_get_full_name (CamelFolder *folder)
 {
 	return folder->full_name;
@@ -263,7 +267,7 @@ digest_get_full_name (CamelFolder *folder)
 
 static void
 digest_append_message (CamelFolder *folder, CamelMimeMessage *message,
-		       const CamelMessageInfo *info, char **appended_uid,
+		       const CamelMessageInfo *info, gchar **appended_uid,
 		       CamelException *ex)
 {
 	/* no-op */
@@ -282,14 +286,14 @@ digest_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 }
 
 static CamelMimeMessage *
-digest_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
+digest_get_message (CamelFolder *folder, const gchar *uid, CamelException *ex)
 {
 	CamelDigestFolder *digest = CAMEL_DIGEST_FOLDER (folder);
 	CamelDataWrapper *wrapper;
 	CamelMimeMessage *message;
 	CamelMimePart *part;
-	char *subuid;
-	int id;
+	gchar *subuid;
+	gint id;
 
 	part = CAMEL_MIME_PART (digest->priv->message);
 	wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
@@ -314,7 +318,7 @@ digest_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 }
 
 static GPtrArray *
-digest_search_by_expression (CamelFolder *folder, const char *expression, CamelException *ex)
+digest_search_by_expression (CamelFolder *folder, const gchar *expression, CamelException *ex)
 {
 	CamelDigestFolder *df = (CamelDigestFolder *) folder;
 	GPtrArray *matches;
@@ -333,7 +337,7 @@ digest_search_by_expression (CamelFolder *folder, const char *expression, CamelE
 }
 
 static GPtrArray *
-digest_search_by_uids (CamelFolder *folder, const char *expression, GPtrArray *uids, CamelException *ex)
+digest_search_by_uids (CamelFolder *folder, const gchar *expression, GPtrArray *uids, CamelException *ex)
 {
 	CamelDigestFolder *df = (CamelDigestFolder *) folder;
 	GPtrArray *matches;
@@ -347,7 +351,7 @@ digest_search_by_uids (CamelFolder *folder, const char *expression, GPtrArray *u
 		df->priv->search = camel_folder_search_new ();
 
 	camel_folder_search_set_folder (df->priv->search, folder);
-	matches = camel_folder_search_search(df->priv->search, expression, NULL, ex);
+	matches = camel_folder_search_search(df->priv->search, expression, uids, ex);
 
 	CAMEL_DIGEST_FOLDER_UNLOCK (folder, search_lock);
 
