@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@novell.com>
  *
- *  Copyright 2005 Novell, Inc. (www.novell.com)
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,6 @@
  *
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -33,7 +32,6 @@
 #include "camel-offline-store.h"
 #include "camel-session.h"
 
-
 static void camel_offline_store_class_init (CamelOfflineStoreClass *klass);
 static void camel_offline_store_init (CamelOfflineStore *store, CamelOfflineStoreClass *klass);
 static void camel_offline_store_finalize (CamelObject *object);
@@ -42,15 +40,13 @@ static void offline_store_construct (CamelService *service, CamelSession *sessio
 				     CamelProvider *provider, CamelURL *url,
 				     CamelException *ex);
 
-
 static CamelStoreClass *parent_class = NULL;
 
-
 CamelType
 camel_offline_store_get_type (void)
 {
 	static CamelType type = NULL;
-	
+
 	if (!type) {
 		type = camel_type_register (CAMEL_STORE_TYPE,
 					    "CamelOfflineStore",
@@ -61,16 +57,15 @@ camel_offline_store_get_type (void)
 					    (CamelObjectInitFunc) camel_offline_store_init,
 					    (CamelObjectFinalizeFunc) camel_offline_store_finalize);
 	}
-	
+
 	return type;
 }
 
-
 static void
 camel_offline_store_class_init (CamelOfflineStoreClass *klass)
 {
 	parent_class = (CamelStoreClass *) camel_type_get_global_classfuncs (CAMEL_STORE_TYPE);
-	
+
 	((CamelServiceClass *) klass)->construct = offline_store_construct;
 }
 
@@ -86,22 +81,34 @@ camel_offline_store_finalize (CamelObject *object)
 	;
 }
 
-
 static void
 offline_store_construct (CamelService *service, CamelSession *session,
 			  CamelProvider *provider, CamelURL *url,
 			  CamelException *ex)
 {
 	CamelOfflineStore *store = CAMEL_OFFLINE_STORE (service);
-	
+
 	CAMEL_SERVICE_CLASS (parent_class)->construct (service, session, provider, url, ex);
 	if (camel_exception_is_set (ex))
 		return;
-	
+
 	store->state = camel_session_is_online (session) ?
 		CAMEL_OFFLINE_STORE_NETWORK_AVAIL : CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL;
 }
 
+/**
+ * camel_offline_store_get_network_state:
+ * @store: a #CamelOfflineStore object
+ * @ex: a #CamelException
+ *
+ * Return the network state either #CAMEL_OFFLINE_STORE_NETWORK_AVAIL
+ * or #CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL.
+ **/
+gint
+camel_offline_store_get_network_state (CamelOfflineStore *store, CamelException *ex)
+{
+	return store->state;
+}
 
 /**
  * camel_offline_store_set_network_state:
@@ -113,7 +120,7 @@ offline_store_construct (CamelService *service, CamelSession *session,
  * or #CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL.
  **/
 void
-camel_offline_store_set_network_state (CamelOfflineStore *store, int state, CamelException *ex)
+camel_offline_store_set_network_state (CamelOfflineStore *store, gint state, CamelException *ex)
 {
 	CamelException lex;
 	CamelService *service = CAMEL_SERVICE (store);
@@ -121,7 +128,7 @@ camel_offline_store_set_network_state (CamelOfflineStore *store, int state, Came
 
 	if (store->state == state)
 		return;
-	
+
 	camel_exception_init (&lex);
 	if (store->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL) {
 		/* network available -> network unavailable */
@@ -129,26 +136,26 @@ camel_offline_store_set_network_state (CamelOfflineStore *store, int state, Came
 			if (((CamelStore *) store)->folders) {
 				GPtrArray *folders;
 				CamelFolder *folder;
-				int i, sync;
-				
+				gint i, sync;
+
 				sync = camel_url_get_param (((CamelService *) store)->url, "sync_offline") != NULL;
-				
+
 				folders = camel_object_bag_list (((CamelStore *) store)->folders);
 				for (i = 0; i < folders->len; i++) {
 					folder = folders->pdata[i];
-					
+
 					if (CAMEL_CHECK_TYPE (folder, CAMEL_OFFLINE_FOLDER_TYPE)
 					    && (sync || ((CamelOfflineFolder *) folder)->sync_offline)) {
 						camel_offline_folder_downsync ((CamelOfflineFolder *) folder, NULL, &lex);
 						camel_exception_clear (&lex);
 					}
-					
+
 					camel_object_unref (folder);
 				}
-				
+
 				g_ptr_array_free (folders, TRUE);
 			}
-			
+
 			camel_store_sync (CAMEL_STORE (store), FALSE, &lex);
 			camel_exception_clear (&lex);
 		}
@@ -156,11 +163,14 @@ camel_offline_store_set_network_state (CamelOfflineStore *store, int state, Came
 		if (!camel_service_disconnect (CAMEL_SERVICE (store), network_state, ex))
 			return;
 	} else {
+		store->state = state;
 		/* network unavailable -> network available */
-		if (!camel_service_connect (CAMEL_SERVICE (store), ex))
+		if (!camel_service_connect (CAMEL_SERVICE (store), ex)) {
+			store->state = CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL;
 			return;
+		}
 	}
-	
+
 	store->state = state;
 }
 
@@ -172,15 +182,15 @@ camel_offline_store_prepare_for_offline (CamelOfflineStore *store, CamelExceptio
 	gboolean network_state = camel_session_get_network_state (service->session);
 
 	camel_exception_init (&lex);
-	if (network_state) {	
+	if (network_state) {
 		if (store->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL) {
 			if (((CamelStore *) store)->folders) {
 				GPtrArray *folders;
 				CamelFolder *folder;
-				int i, sync;
-				
+				gint i, sync;
+
 				sync = camel_url_get_param (((CamelService *) store)->url, "sync_offline") != NULL;
-				
+
 				folders = camel_object_bag_list (((CamelStore *) store)->folders);
 				for (i = 0; i < folders->len; i++) {
 					folder = folders->pdata[i];
@@ -194,10 +204,10 @@ camel_offline_store_prepare_for_offline (CamelOfflineStore *store, CamelExceptio
 				}
 				g_ptr_array_free (folders, TRUE);
 			}
-		}	
+		}
 
 		camel_store_sync (CAMEL_STORE (store), FALSE, &lex);
 		camel_exception_clear (&lex);
-	
+
 	}
 }
