@@ -40,16 +40,17 @@
 #include "camel-private.h"
 
 #include "camel-mh-summary.h"
+#include "camel-local-private.h"
 
 #define d(x) /*(printf("%s(%d): ", __FILE__, __LINE__),(x))*/
 
 #define CAMEL_MH_SUMMARY_VERSION (0x2000)
 
-static int mh_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex);
-static int mh_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex);
-/*static int mh_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, CamelMessageInfo *info, CamelFolderChangeInfo *, CamelException *ex);*/
+static gint mh_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex);
+static gint mh_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changeinfo, CamelException *ex);
+/*static gint mh_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, CamelMessageInfo *info, CamelFolderChangeInfo *, CamelException *ex);*/
 
-static char *mh_summary_next_uid_string(CamelFolderSummary *s);
+static gchar *mh_summary_next_uid_string(CamelFolderSummary *s);
 
 static void camel_mh_summary_class_init	(CamelMhSummaryClass *class);
 static void camel_mh_summary_init	(CamelMhSummary *gspaper);
@@ -58,7 +59,7 @@ static void camel_mh_summary_finalise	(CamelObject *obj);
 #define _PRIVATE(x) (((CamelMhSummary *)(x))->priv)
 
 struct _CamelMhSummaryPrivate {
-	char *current_uid;
+	gchar *current_uid;
 };
 
 static CamelLocalSummaryClass *parent_class;
@@ -67,7 +68,7 @@ CamelType
 camel_mh_summary_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register(camel_local_summary_get_type (), "CamelMhSummary",
 					   sizeof(CamelMhSummary),
@@ -77,7 +78,7 @@ camel_mh_summary_get_type (void)
 					   (CamelObjectInitFunc)camel_mh_summary_init,
 					   (CamelObjectFinalizeFunc)camel_mh_summary_finalise);
 	}
-	
+
 	return type;
 }
 
@@ -115,44 +116,20 @@ camel_mh_summary_finalise(CamelObject *obj)
 	g_free(o->priv);
 }
 
-static int 
-sort_uid_cmp (void *enc, int len1, void * data1, int len2, void *data2)
-{
-	static char *sa1=NULL, *sa2=NULL;
-	static int l1=0, l2=0;
-	int a1, a2;
-
-	if (l1 < len1+1) {
-		sa1 = g_realloc (sa1, len1+1);
-		l1 = len1+1;
-	}
-	if (l2 < len2+1) {
-		sa2 = g_realloc (sa2, len2+1);
-		l2 = len2+1;
-	}
-	strncpy (sa1, data1, len1);sa1[len1] = 0;
-	strncpy (sa2, data2, len2);sa2[len2] = 0;	
-
-	a1 = strtoul (sa1, NULL, 10);
-	a2 = strtoul (sa2, NULL, 10);
-
-	return (a1 < a1) ? -1 : (a1 > a2) ? 1 : 0;
-}
-
 /**
  * camel_mh_summary_new:
  *
  * Create a new CamelMhSummary object.
- * 
+ *
  * Return value: A new #CamelMhSummary object.
  **/
-CamelMhSummary	*camel_mh_summary_new(struct _CamelFolder *folder, const char *filename, const char *mhdir, CamelIndex *index)
+CamelMhSummary	*camel_mh_summary_new(struct _CamelFolder *folder, const gchar *filename, const gchar *mhdir, CamelIndex *index)
 {
 	CamelMhSummary *o = (CamelMhSummary *)camel_object_new(camel_mh_summary_get_type ());
 
 	((CamelFolderSummary *)o)->folder = folder;
 	if (folder) {
-		camel_db_set_collate (folder->parent_store->cdb_r, "uid", "mh_uid_sort", (CamelDBCollate)sort_uid_cmp);
+		camel_db_set_collate (folder->parent_store->cdb_r, "uid", "mh_uid_sort", (CamelDBCollate)camel_local_frompos_sort);
 		((CamelFolderSummary *)o)->sort_by = "uid";
 		((CamelFolderSummary *)o)->collate = "mh_uid_sort";
 	}
@@ -161,14 +138,14 @@ CamelMhSummary	*camel_mh_summary_new(struct _CamelFolder *folder, const char *fi
 	return o;
 }
 
-static char *mh_summary_next_uid_string(CamelFolderSummary *s)
+static gchar *mh_summary_next_uid_string(CamelFolderSummary *s)
 {
 	CamelMhSummary *mhs = (CamelMhSummary *)s;
 	CamelLocalSummary *cls = (CamelLocalSummary *)s;
-	int fd = -1;
+	gint fd = -1;
 	guint32 uid;
-	char *name;
-	char *uidstr;
+	gchar *name;
+	gchar *uidstr;
 
 	/* if we are working to add an existing file, then use current_uid */
 	if (mhs->priv->current_uid) {
@@ -196,18 +173,18 @@ static char *mh_summary_next_uid_string(CamelFolderSummary *s)
 	return uidstr;
 }
 
-static int camel_mh_summary_add(CamelLocalSummary *cls, const char *name, int forceindex)
+static gint camel_mh_summary_add(CamelLocalSummary *cls, const gchar *name, gint forceindex)
 {
 	CamelMhSummary *mhs = (CamelMhSummary *)cls;
-	char *filename = g_strdup_printf("%s/%s", cls->folder_path, name);
-	int fd;
+	gchar *filename = g_strdup_printf("%s/%s", cls->folder_path, name);
+	gint fd;
 	CamelMimeParser *mp;
 
 	d(printf("summarising: %s\n", name));
 
 	fd = open(filename, O_RDONLY|O_LARGEFILE);
 	if (fd == -1) {
-		g_warning ("Cannot summarise/index: %s: %s", filename, strerror (errno));
+		g_warning ("Cannot summarise/index: %s: %s", filename, g_strerror (errno));
 		g_free(filename);
 		return -1;
 	}
@@ -220,7 +197,7 @@ static int camel_mh_summary_add(CamelLocalSummary *cls, const char *name, int fo
 	} else {
 		camel_folder_summary_set_index((CamelFolderSummary *)mhs, NULL);
 	}
-	mhs->priv->current_uid = (char *)name;
+	mhs->priv->current_uid = (gchar *)name;
 	camel_folder_summary_add_from_parser((CamelFolderSummary *)mhs, mp);
 	camel_object_unref((CamelObject *)mp);
 	mhs->priv->current_uid = NULL;
@@ -230,7 +207,7 @@ static int camel_mh_summary_add(CamelLocalSummary *cls, const char *name, int fo
 }
 
 static void
-remove_summary(char *key, CamelMessageInfo *info, CamelLocalSummary *cls)
+remove_summary(gchar *key, CamelMessageInfo *info, CamelLocalSummary *cls)
 {
 	d(printf("removing message %s from summary\n", key));
 	if (cls->index)
@@ -239,18 +216,17 @@ remove_summary(char *key, CamelMessageInfo *info, CamelLocalSummary *cls)
 	camel_message_info_free(info);
 }
 
-
-static int
+static gint
 mh_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, CamelException *ex)
 {
 	DIR *dir;
 	struct dirent *d;
-	char *p, c;
+	gchar *p, c;
 	CamelMessageInfo *info;
 	CamelFolderSummary *s = (CamelFolderSummary *)cls;
 	GHashTable *left;
-	int i, count;
-	int forceindex;
+	gint i, count;
+	gint forceindex;
 
 	/* FIXME: Handle changeinfo */
 
@@ -273,7 +249,7 @@ mh_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, Came
 	for (i=0;i<count;i++) {
 		info = camel_folder_summary_index((CamelFolderSummary *)cls, i);
 		if (info) {
-			g_hash_table_insert(left, (char *)camel_message_info_uid(info), info);
+			g_hash_table_insert(left, (gchar *)camel_message_info_uid(info), info);
 		}
 	}
 
@@ -295,7 +271,7 @@ mh_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, Came
 				}
 				camel_mh_summary_add(cls, d->d_name, forceindex);
 			} else {
-				const char *uid = camel_message_info_uid(info);
+				const gchar *uid = camel_message_info_uid(info);
 				CamelMessageInfo *old = g_hash_table_lookup(left, uid);
 
 				if (old) {
@@ -318,13 +294,13 @@ mh_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changeinfo, Came
 }
 
 /* sync the summary file with the ondisk files */
-static int
+static gint
 mh_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo *changes, CamelException *ex)
 {
-	int count, i;
+	gint count, i;
 	CamelLocalMessageInfo *info;
-	char *name;
-	const char *uid;
+	gchar *name;
+	const gchar *uid;
 
 	d(printf("summary_sync(expunge=%s)\n", expunge?"true":"false"));
 
@@ -347,8 +323,8 @@ mh_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInfo
 
 				/* FIXME: put this in folder_summary::remove()? */
 				if (cls->index)
-					camel_index_delete_name(cls->index, (char *)uid);
-				
+					camel_index_delete_name(cls->index, (gchar *)uid);
+
 				camel_folder_change_info_remove_uid(changes, uid);
 				camel_folder_summary_remove((CamelFolderSummary *)cls, (CamelMessageInfo *)info);
 			}
