@@ -10,14 +10,14 @@
 
 #include "camel/camel-stream-mem.h"
 
-int main(int argc, char **argv)
+gint main(gint argc, gchar **argv)
 {
 	CamelSeekableStream *ss = NULL;
-	int i;
-	int fd = -1;
+	gint i;
+	gint fd = -1;
 	struct stat st;
-	int size;
-	char buffer[1024];
+	gint size;
+	gchar buffer[1024];
 	GByteArray *ba;
 
 	camel_test_init(argc, argv);
@@ -26,7 +26,7 @@ int main(int argc, char **argv)
 	for (i=0;i<3;i++) {
 
 		push("Creating stream using method %d", i);
-		switch(i) {
+		switch (i) {
 		case 0:
 			ss = (CamelSeekableStream *)camel_stream_mem_new();
 			break;
