@@ -172,7 +172,7 @@ e_name_selector_entry_class_init (ENameSelectorEntryClass *name_selector_entry_c
 	g_type_class_add_private (object_class, sizeof(ENameSelectorEntryPrivate));
 }
 
-/* Remove unquoted commas from string */
+/* Remove unquoted commas and control characters from string */
 static gchar *
 sanitize_string (const gchar *string)
 {
@@ -192,6 +192,8 @@ sanitize_string (const gchar *string)
 			quoted = ~quoted;
 		else if (c == ',' && !quoted)
 			continue;
+		else if (c == '\t' || c == '\n')
+			continue;
 
 		g_string_append_unichar (gstring, c);
 	}
@@ -1386,7 +1388,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 		p = text + end_pos;
 		for (p = text + (end_pos-1), j = end_pos - 1; *p && *p != '"' ; p = g_utf8_next_char (p), j++) {
 			gunichar c = g_utf8_get_char (p);
-			if(c == ',') {
+			if (c == ',') {
 				insert_destination_at_position (name_selector_entry, j+1);
 			}
 		}
@@ -1399,7 +1401,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 		/* If the entry was completely cleared, remove the initial destination too */
 		remove_destination_by_index (name_selector_entry, 0);
 		generate_attribute_list (name_selector_entry);
-	} else  if (!del_space){
+	} else  if (!del_space) {
 		modify_destination_at_position (name_selector_entry, start_pos);
 	}
 
@@ -1576,11 +1578,48 @@ sanitize_entry (ENameSelectorEntry *name_selector_entry)
 static gboolean
 user_focus_in (ENameSelectorEntry *name_selector_entry, GdkEventFocus *event_focus)
 {
-	/*
-	 * To preserve selected text, do not propagate the event any more.
-	 */
+	gint n;
+	GList *l, *known;
+	GString *str = g_string_new ("");
+	EDestination *dest_dummy = e_destination_new ();
 
-	return TRUE;
+	g_signal_handlers_block_matched (name_selector_entry, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, name_selector_entry);
+	g_signal_handlers_block_matched (name_selector_entry->destination_store, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, name_selector_entry);
+
+	known = e_destination_store_list_destinations (name_selector_entry->destination_store);
+	for (l = known, n = 0; l != NULL; l = l->next, n++) {
+		EDestination *dest = l->data;
+
+		if (dest) {
+			gchar *text;
+
+			text = get_destination_textrep (dest);
+			if (text) {
+				if (str->str && str->str[0])
+					g_string_append (str, ", ");
+
+				g_string_append (str, text);
+			}
+			g_free (text);
+		}
+	}
+	g_list_free (known);
+
+	/* Add a blank destination */
+	e_destination_store_append_destination (name_selector_entry->destination_store, dest_dummy);
+	if (str->str && str->str[0])
+		g_string_append (str, ", ");
+
+	gtk_entry_set_text (GTK_ENTRY (name_selector_entry), str->str);
+
+	g_string_free (str, TRUE);
+
+	g_signal_handlers_unblock_matched (name_selector_entry->destination_store, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, name_selector_entry);
+	g_signal_handlers_unblock_matched (name_selector_entry, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, name_selector_entry);
+
+	generate_attribute_list (name_selector_entry);
+
+	return FALSE;
 }
 
 static gboolean
