@@ -2,7 +2,7 @@
 /* test-source-list-selector.c - Test program for the ESourceListSelector
  * widget.
  *
- * Copyright (C) 2002 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -27,11 +27,7 @@
 
 #include "e-source-selector.h"
 
-#include <gtk/gtkmain.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 
 static void
 dump_selection (ESourceSelector *selector)
@@ -58,7 +54,7 @@ dump_selection (ESourceSelector *selector)
 
 static void
 selection_changed_callback (ESourceSelector *selector,
-			    void *unused_data)
+			    gpointer unused_data)
 {
 	g_print ("Selection changed!\n");
 	dump_selection (selector);
@@ -71,8 +67,8 @@ check_toggled_callback (GtkToggleButton *button,
 	e_source_selector_show_selection (selector, gtk_toggle_button_get_active (button));
 }
 
-static int
-on_idle_create_widget (const char *gconf_path)
+static gint
+on_idle_create_widget (const gchar *gconf_path)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
@@ -111,11 +107,10 @@ on_idle_create_widget (const char *gconf_path)
 	return FALSE;
 }
 
-
-int
-main (int argc, char **argv)
+gint
+main (gint argc, gchar **argv)
 {
-	const char *gconf_path;
+	const gchar *gconf_path;
 
 	gtk_init (&argc, &argv);
 	if (argc < 2)
@@ -123,7 +118,7 @@ main (int argc, char **argv)
 	else
 		gconf_path = argv [1];
 
-	g_idle_add ((GSourceFunc) on_idle_create_widget, (void *) gconf_path);
+	g_idle_add ((GSourceFunc) on_idle_create_widget, (gpointer) gconf_path);
 
 	gtk_main ();
 
