@@ -5,12 +5,9 @@
 #define __EXCHANGE_HIERARCHY_WEBDAV_H__
 
 #include "exchange-hierarchy.h"
-//#include "exchange-folder-size.h"
+/* #include "exchange-folder-size.h" */
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 #define EXCHANGE_TYPE_HIERARCHY_WEBDAV            (exchange_hierarchy_webdav_get_type ())
 #define EXCHANGE_HIERARCHY_WEBDAV(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXCHANGE_TYPE_HIERARCHY_WEBDAV, ExchangeHierarchyWebDAV))
@@ -33,12 +30,12 @@ GType              exchange_hierarchy_webdav_get_type (void);
 
 ExchangeHierarchy *exchange_hierarchy_webdav_new (ExchangeAccount *account,
 						  ExchangeHierarchyType type,
-						  const char *hierarchy_name,
-						  const char *physical_uri_prefix,
-						  const char *internal_uri_prefix,
-						  const char *owner_name,
-						  const char *owner_email,
-						  const char *source_uri,
+						  const gchar *hierarchy_name,
+						  const gchar *physical_uri_prefix,
+						  const gchar *internal_uri_prefix,
+						  const gchar *owner_name,
+						  const gchar *owner_email,
+						  const gchar *source_uri,
 						  gboolean deep_searchable);
 
 /* for subclasses */
@@ -50,12 +47,12 @@ EFolder *exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 void exchange_hierarchy_webdav_construct   (ExchangeHierarchyWebDAV *hwd,
 					    ExchangeAccount *account,
 					    ExchangeHierarchyType type,
-					    const char *hierarchy_name,
-					    const char *physical_uri_prefix,
-					    const char *internal_uri_prefix,
-					    const char *owner_name,
-					    const char *owner_email,
-					    const char *source_uri,
+					    const gchar *hierarchy_name,
+					    const gchar *physical_uri_prefix,
+					    const gchar *internal_uri_prefix,
+					    const gchar *owner_name,
+					    const gchar *owner_email,
+					    const gchar *source_uri,
 					    gboolean deep_searchable);
 
 typedef void (*ExchangeHierarchyWebDAVScanCallback)    (ExchangeHierarchy *hier,
@@ -67,9 +64,6 @@ void    exchange_hierarchy_webdav_offline_scan_subtree (ExchangeHierarchy *hier,
 
 gdouble exchange_hierarchy_webdav_get_total_folder_size (ExchangeHierarchyWebDAV *hwd);
 
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __EXCHANGE_HIERARCHY_WEBDAV_H__ */
